//
//  DTKViewController.m
//  IOEffects
//
//  Created by Chris Adamson on 4/19/13.
//  Copyright (c) 2013 Chris Adamson. All rights reserved.
//

#import "DTKViewController.h"
#import <AudioToolbox/AudioToolbox.h>

@interface DTKViewController ()
@property (assign) AUGraph auGraph;
@property (assign) AudioUnit effectUnit;
@property (assign) AudioUnit remoteIOUnit;
- (IBAction)sliderValueChanged:(UISlider *)sender;
@end

@implementation DTKViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
	[self buildAUGraph];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

-(void) buildAUGraph {
	OSStatus err = noErr;
	
	err = NewAUGraph(&_auGraph);
	if (err) { NSLog (@"couldn't create AUGraph"); return; }
	
	err = AUGraphOpen(self.auGraph);
	if (err) { NSLog (@"couldn't open AUGRaph"); return; }
	
	AUNode remoteIONode;
	
	AudioComponentDescription compDesc;
	compDesc.componentType = kAudioUnitType_Output;
	compDesc.componentSubType = kAudioUnitSubType_RemoteIO;
	compDesc.componentManufacturer = kAudioUnitManufacturer_Apple;
	
	err = AUGraphAddNode(self.auGraph,
						 &compDesc,
						 &remoteIONode);
	if (err) { NSLog (@"couldn't create remote io node"); return; }

	compDesc.componentType = kAudioUnitType_Effect;
	compDesc.componentSubType = kAudioUnitSubType_Reverb2;
	compDesc.componentManufacturer = kAudioUnitManufacturer_Apple;
	
	AUNode effectNode;
	err = AUGraphAddNode(self.auGraph,
						 &compDesc,
						 &effectNode);
	if (err) { NSLog (@"couldn't create effect node"); return; }
	
	err = AUGraphNodeInfo(self.auGraph,
						  effectNode,
						  NULL,
						  &_effectUnit);
	if (err) { NSLog (@"couldn't get effect unit from node"); return; }
	
	AudioStreamBasicDescription streamFormat;
	UInt32 propSize = sizeof (streamFormat);
	err = AudioUnitGetProperty(self.effectUnit,
							   kAudioUnitProperty_StreamFormat,
							   kAudioUnitScope_Output,
							   0,
							   &streamFormat,
							   &propSize);
	if (err) { NSLog (@"couldn't get effect unit format"); return; }
	
	err = AUGraphNodeInfo(self.auGraph,
						  remoteIONode,
						  NULL,
						  &_remoteIOUnit);
	if (err) { NSLog (@"couldn't get remote io unit from node"); return ; }
	err = AudioUnitSetProperty(self.remoteIOUnit,
							   kAudioUnitProperty_StreamFormat,
							   kAudioUnitScope_Output,
							   1,
							   &streamFormat,
							   sizeof(streamFormat));
	if (err) {NSLog (@"couldn't set format on remote io output bus 1"); return; }
	err = AudioUnitSetProperty(self.remoteIOUnit,
							   kAudioUnitProperty_StreamFormat,
							   kAudioUnitScope_Input,
							   0,
							   &streamFormat,
							   sizeof(streamFormat));
	if (err) {NSLog (@"couldn't set format on remote io input bus 0"); return; }

	UInt32 flag = 1;
	err = AudioUnitSetProperty(self.remoteIOUnit,
							   kAudioOutputUnitProperty_EnableIO,
							   kAudioUnitScope_Input,
							   1,
							   &flag,
							   sizeof(flag));
	if (err) {NSLog (@"couldn't enable remote io input"); return ; }
	err = AudioUnitSetProperty(self.remoteIOUnit,
							   kAudioOutputUnitProperty_EnableIO,
							   kAudioUnitScope_Output,
							   0,
							   &flag,
							   sizeof(flag));
	
	err = AUGraphConnectNodeInput(self.auGraph,
								  remoteIONode,
								  1,
								  effectNode,
								  0);
	if (err) { NSLog (@"couldn't connect remote io bus 1 to effect bus 0"); return;}
	err = AUGraphConnectNodeInput(self.auGraph,
								  effectNode,
								  0,
								  remoteIONode,
								  0);
	if (err) { NSLog (@"couldn't connect remote io bus 1 to effect bus 0"); return;}
		
	err = AUGraphInitialize(self.auGraph);
	if (err) { NSLog (@"couldn't initialize graph"); return; }
	
	err = AUGraphStart(self.auGraph);
	if (err) { NSLog (@"couldn't start graph"); return; }
		
}


- (IBAction)sliderValueChanged:(UISlider *)sender {
	OSStatus err = AudioUnitSetParameter(self.effectUnit,
										 kReverb2Param_DryWetMix,
										 kAudioUnitScope_Global,
										 0,
										 sender.value,
										 0);
	if (err) { NSLog (@"couldn't set reverb dry/wet mix value"); return; }
}
@end
